@@ -14,6 +14,9 @@
 #' - "LinDA": Linear models for differential abundance analysis of microbiome compositional data
 #' - "Maaslin2": Multivariate Association with Linear Models (MaAsLin2) for differential abundance analysis
 #' @param ko_to_kegg A character to control the conversion of KO abundance to KEGG abundance
+#' @param filter_for_prokaryotes Logical. If TRUE (default), filters out KEGG pathways
+#'   that are specific to eukaryotes (e.g., human diseases, organismal systems) when
+#'   ko_to_kegg = TRUE. Set to FALSE to include all KEGG pathways.
 #' @param p.adjust a character specifying the method for p-value adjustment, default is "BH", choices are:
 #'- "BH": Benjamini-Hochberg correction
 #'- "holm": Holm's correction
@@ -27,10 +30,25 @@
 #' @param select A vector consisting of pathway names to be selected
 #' @param reference A character, a reference group level for several DA methods
 #' @param colors A vector consisting of colors number
+#' @param p_values_threshold A numeric value specifying the threshold for statistical
+#'   significance of differential abundance. Pathways with adjusted p-values below this
+#'   threshold will be displayed in the plot. Default is 0.05.
 #'
-#' @return daa.results.df, a dataframe of DA results
-#' A list of sub-lists, each containing a ggplot2 plot (`plot`) and a dataframe of differential abundance results (`results`) for a specific DA method.
-#' Each plot visualizes the differential abundance results of a specific DA method, and the corresponding dataframe contains the results used to create the plot.
+#' @return A list containing:
+#' \itemize{
+#'   \item Numbered elements (1, 2, ...): Sub-lists for each DA method, each containing:
+#'     \itemize{
+#'       \item \code{plot}: A ggplot2 error bar plot visualizing the differential abundance results
+#'       \item \code{results}: A data frame of differential abundance results for that method
+#'     }
+#'   \item \code{abundance}: The processed abundance data (KEGG pathway or original) for downstream analysis
+#'   \item \code{metadata}: The metadata data frame
+#'   \item \code{group}: The group variable name used in the analysis
+#'   \item \code{daa_results_df}: The complete annotated DAA results data frame
+#'   \item \code{ko_to_kegg}: Logical indicating whether KO to KEGG conversion was performed
+#' }
+#' These additional fields allow seamless integration with \code{\link{pathway_pca}} and
+#' \code{\link{pathway_heatmap}} for further visualization without re-preparing data.
 #' @export
 #' @examples
 #' \dontrun{
@@ -90,6 +108,25 @@
 #'                                  order = "group",
 #'                                  p_values_bar = TRUE,
 #'                                  x_lab = "description")
+#'
+#' # Use the returned data for PCA analysis (no need to re-prepare data)
+#' pca_plot <- pathway_pca(
+#'   abundance = results_file_input$abundance,
+#'   metadata = results_file_input$metadata,
+#'   group = results_file_input$group
+#' )
+#'
+#' # Use the returned data for heatmap (filter significant pathways first)
+#' sig_features <- results_file_input$daa_results_df %>%
+#'   dplyr::filter(p_adjust < 0.05) %>%
+#'   dplyr::pull(feature)
+#' if (length(sig_features) > 0) {
+#'   heatmap_plot <- pathway_heatmap(
+#'     abundance = results_file_input$abundance[sig_features, , drop = FALSE],
+#'     metadata = results_file_input$metadata,
+#'     group = results_file_input$group
+#'   )
+#' }
 #'}
 ggpicrust2 <- function(file = NULL,
                        data = NULL,
@@ -98,13 +135,15 @@ ggpicrust2 <- function(file = NULL,
                        pathway,
                        daa_method = "ALDEx2",
                        ko_to_kegg = FALSE,
+                       filter_for_prokaryotes = TRUE,
                        p.adjust = "BH",
                        order = "group",
                        p_values_bar = TRUE,
                        x_lab = NULL,
                        select = NULL,
                        reference = NULL,
-                       colors = NULL) {
+                       colors = NULL,
+                       p_values_threshold = 0.05) {
   # Input validation
   if (is.null(file) && is.null(data)) {
     stop("Error: Please provide either a 'file' path (character string) or a 'data' data frame.")
@@ -132,166 +171,110 @@ ggpicrust2 <- function(file = NULL,
     }
   }
 
-  # Create an empty list
-  plot_result_list <- list()
+  # Validate daa_method early
 
-  message("Starting the ggpicrust2 analysis...\n")
+  if (daa_method == "Lefse") {
+    stop("The 'Lefse' method is not suitable for ggpicrust2() as it does not output p-values.")
+  }
 
-  if (ko_to_kegg == TRUE){
+
+  # Step 1: Load abundance data
+  if (ko_to_kegg) {
     message("Converting KO to KEGG...\n")
-    plot_result_list <- list()
     abundance <- if (!is.null(file)) {
-      ko2kegg_abundance(file)
+      ko2kegg_abundance(file, filter_for_prokaryotes = filter_for_prokaryotes)
     } else {
-      ko2kegg_abundance(data = data)
+      ko2kegg_abundance(data = data, filter_for_prokaryotes = filter_for_prokaryotes)
     }
-    message("Performing pathway differential abundance analysis...\n")
-    daa_results_df <- pathway_daa(
-      abundance = abundance,
-      metadata = metadata,
-      group = group,
-      daa_method = daa_method,
-      select = select,
-      p.adjust = p.adjust,
-      reference = reference
-    )
-    if(daa_method == "Lefse") {
-      stop("The 'Lefse' method is not suitable for the ggpicrust2() function as Lefse in R does not output p-values, only effect sizes.")
-    }
-
-    # Checking for statistically significant biomarkers in the dataset
-    num_significant_biomarkers <- sum(as.numeric(daa_results_df$p_adjust <= 0.05))
-
-    if (num_significant_biomarkers == 0) {
-      # If no biomarkers have p-values less than or equal to 0.05, issue a warning but continue processing
-      warning(
-        "No statistically significant biomarkers found in the dataset (p_adjust <= 0.05). ",
-        "This is not an error, but indicates that the data do not contain biomarkers passing the significance threshold. ",
-        "The analysis will continue with empty annotation columns for visualization purposes. ",
-        "You may refer to the tutorial's FAQ for further help and suggestions.",
-        call. = FALSE
-      )
-      message("Notice: Continuing analysis with non-significant results for visualization...")
-    } else {
-      message(paste("Success: Found", num_significant_biomarkers, "statistically significant biomarker(s) in the dataset."))
-    }
-
-    message("Annotating pathways...\n")
-    daa_results_df <-
-      pathway_annotation(
-        pathway = pathway,
-        ko_to_kegg = ko_to_kegg,
-        daa_results_df = daa_results_df
-      )
-    j <- 1
-    message("Creating pathway error bar plots...\n")
-    for (i in unique(daa_results_df$method)) {
-      daa_sub_method_results_df <-
-        daa_results_df[daa_results_df[, "method"] == i, ]
-      combination_bar_plot <-
-        pathway_errorbar(
-          abundance = abundance,
-          daa_results_df = daa_sub_method_results_df,
-          Group = metadata %>% select(all_of(c(group))) %>% as.data.frame() %>% pull(),
-          ko_to_kegg = ko_to_kegg,
-          p_value_bar = p_values_bar,
-          order = order,
-          colors = colors,
-          select = select,
-          x_lab = x_lab
-        )
-      # Create a sublist containing a combination_bar_plot and the corresponding subset of daa_results_df
-      sub_list <-
-        list(plot = combination_bar_plot, results = daa_sub_method_results_df)
-      # Add sublists to the main list
-      plot_result_list[[j]] <- sub_list
-      message(sprintf("Plot %d created.\n", j))
-      j <- j + 1
-    }
-    message("ggpicrust2 analysis completed.\n")
-    return(plot_result_list)
   } else {
-    message("Reading input file or using provided data...\n")
-    plot_result_list <- list()
+    message("Reading input data...\n")
     abundance <- if (!is.null(file)) {
-      readr::read_delim(
-        file,
-        delim = "\t",
-        escape_double = FALSE,
-        trim_ws = TRUE
-      )
+      read_abundance_file(file)
     } else {
       data
     }
     abundance <- as.data.frame(abundance)
-    
-    # PICRUSt 2.6.2兼容性处理：清理KO ID格式
-    ko_ids <- abundance[, 1]
-    has_ko_prefix <- any(grepl("^ko:", ko_ids))
-    if (has_ko_prefix) {
-      message("Detected PICRUSt 2.6.2 format with 'ko:' prefix. Applying compatibility fix...")
-      abundance[, 1] <- gsub("^ko:", "", abundance[, 1])
-      message(sprintf("Cleaned %d KO IDs by removing 'ko:' prefix", sum(grepl("^ko:", ko_ids))))
-    }
-    
+    abundance <- clean_ko_abundance(abundance)
     rownames(abundance) <- abundance[, 1]
     abundance <- abundance[, -1]
-    message("Performing pathway differential abundance analysis...\n")
-    daa_results_df <- pathway_daa(
-      abundance = abundance,
-      metadata = metadata,
-      group = group,
-      daa_method = daa_method,
-      select = select,
-      p.adjust = p.adjust,
-      reference = reference
-    )
-    if(daa_method == "Lefse") {
-      stop("The 'Lefse' method is not suitable for the ggpicrust2() function as Lefse in R does not output p-values, only effect sizes.")
-    }
-    message("Annotating pathways...\n")
-    daa_results_df <-
-      pathway_annotation(
-        pathway = pathway,
-        ko_to_kegg = ko_to_kegg,
-        daa_results_df = daa_results_df
-      )
-    j <- 1
-    message("Creating pathway error bar plots...\n")
-    for (i in unique(daa_results_df$method)) {
-      daa_sub_method_results_df <-
-        daa_results_df[daa_results_df[, "method"] == i, ]
-      combination_bar_plot <-
-        pathway_errorbar(
-          abundance = abundance,
-          daa_results_df = daa_sub_method_results_df,
-          Group = metadata %>% select(all_of(c(group))) %>% as.data.frame() %>% pull(),
-          ko_to_kegg = ko_to_kegg,
-          p_value_bar = p_values_bar,
-          order = order,
-          colors = colors,
-          select = select,
-          x_lab = x_lab
-        )
-      
-      # Check if pathway_errorbar returned NULL (no data for plotting)
-      if (is.null(combination_bar_plot)) {
-        message(sprintf("Plot %d skipped due to insufficient annotation data for method: %s\n", j, i))
-        # Still create a sublist with results but no plot
-        sub_list <- list(plot = NULL, results = daa_sub_method_results_df)
-      } else {
-        # Create a sublist with both plot and results
-        sub_list <- list(plot = combination_bar_plot, results = daa_sub_method_results_df)
-        message(sprintf("Plot %d created.\n", j))
-      }
-      
-      # Add sublists to the main list
-      plot_result_list[[j]] <- sub_list
-      j <- j + 1
-    }
-    message("ggpicrust2 analysis completed.\n")
-    return(plot_result_list)
   }
 
+  # Align abundance and metadata once for consistent downstream behavior.
+  # This prevents Group/order mismatches in pathway_errorbar() when metadata
+  # row order differs from abundance column order.
+  aligned <- align_samples(abundance, metadata, verbose = FALSE)
+  abundance <- aligned$abundance
+  metadata <- aligned$metadata
 
+  # Step 2: Differential abundance analysis
+  message("Performing pathway differential abundance analysis...\n")
+  daa_results_df <- pathway_daa(
+    abundance = abundance,
+    metadata = metadata,
+    group = group,
+    daa_method = daa_method,
+    select = select,
+    p.adjust = p.adjust,
+    reference = reference
+  )
+
+  # Check for significant biomarkers
+  num_significant <- sum(daa_results_df$p_adjust < p_values_threshold, na.rm = TRUE)
+  if (num_significant == 0) {
+    warning(sprintf("No statistically significant biomarkers found (p_adjust < %g). ", p_values_threshold),
+            "Analysis will continue for visualization purposes.", call. = FALSE)
+  }
+
+  # Step 3: Pathway annotation
+  message("Annotating pathways...\n")
+  daa_results_df <- pathway_annotation(
+    pathway = pathway,
+    ko_to_kegg = ko_to_kegg,
+    daa_results_df = daa_results_df,
+    p_adjust_threshold = p_values_threshold
+  )
+
+  # Step 4: Create plots for each method
+  message("Creating pathway error bar plots...\n")
+  plot_result_list <- list()
+  Group_vec <- metadata[[group]]
+  names(Group_vec) <- colnames(abundance)
+
+  for (i in seq_along(unique(daa_results_df$method))) {
+    method_name <- unique(daa_results_df$method)[i]
+    daa_sub_method_results_df <- daa_results_df[daa_results_df$method == method_name, ]
+
+    combination_bar_plot <- pathway_errorbar(
+      abundance = abundance,
+      daa_results_df = daa_sub_method_results_df,
+      Group = Group_vec,
+      ko_to_kegg = ko_to_kegg,
+      p_value_bar = p_values_bar,
+      order = order,
+      colors = colors,
+      select = select,
+      x_lab = x_lab,
+      p_values_threshold = p_values_threshold
+    )
+
+    if (is.null(combination_bar_plot)) {
+      message(sprintf("Plot %d skipped (no data for method: %s)\n", i, method_name))
+    } else {
+      message(sprintf("Plot %d created.\n", i))
+    }
+
+    plot_result_list[[i]] <- list(plot = combination_bar_plot, results = daa_sub_method_results_df)
+  }
+
+  message("ggpicrust2 analysis completed.\n")
+
+  # Step 5: Add data for downstream analysis
+
+  plot_result_list$abundance <- abundance
+  plot_result_list$metadata <- metadata
+  plot_result_list$group <- group
+  plot_result_list$daa_results_df <- daa_results_df
+  plot_result_list$ko_to_kegg <- ko_to_kegg
+
+  return(plot_result_list)
 }
